***********************************************************************
* Class......: ObjBase
* Purpose....: Abstract Class definition for Error Logging Object
***********************************************************************
DEFINE CLASS objBase AS RELATION OLEPUBLIC
    *** Give it an Error Collection
    DIMENSION aErrors[1] 
    *** And a property to store Calling Procedure
    cCalledBy = ""
    
    *******************************************************************
    *** [E] ERROR(): Re-Define the Error Method to Call LogErrors()
    *******************************************************************
    FUNCTION Error( nError, cMethod, nLine )
        *** Log the error
        This.LogError( nError, MESSAGE(), cMethod )
        *** Return an Error Code
        RETURN -1
    ENDFUNC

    *******************************************************************
    *** [P] LOGERROR(): Adds an error to the Errors collection 
    *******************************************************************
    PROTECTED FUNCTION LogError( tnErrNum, tcErrMsg, tcMethod )
        LOCAL lnErrs
        WITH This
            *** How many Errors currently?
            IF EMPTY( .Aerrors[ 1, 1 ] )
                lnErrs = 1
            ELSE
                lnErrs = ALEN( .AErrors, 1 ) + 1
            ENDIF
            *** Re-Dimension the Collection
            DIMENSION .AErrors[ lnErrs, 3 ]
            *** And Populate the current row
            .AErrors[lnErrs, 1] = TRANSFORM( tnErrNum )
            .AErrors[lnErrs, 2] = ALLTRIM( TRANSFORM( tcErrMsg ))
            .AErrors[lnErrs, 3] = ALLTRIM( TRANSFORM( tcMethod ))
        ENDWITH
        *** Just Return out of here
        RETURN .T.
    ENDFUNC

    *******************************************************************
    *** [E] GETERRORS(): Returns object with contents of Errors Collection
    *******************************************************************
    FUNCTION GetErrors 
        LOCAL lnErrCnt, loParamObj, lcErrName, lcErrDets, lnCnt
        *** Create the parameter object
        loParamObj = CREATEOBJECT( 'relation' )
        *** Check if we have any errors
        lnErrCnt = ALEN( This.AErrors, 1 )
        IF lnErrCnt = 1 AND EMPTY( This.AErrors[1,1] )
            *** No Errors Pending
            loParamObj.AddProperty( 'nErrorCount', 0 )
        ELSE
            *** Populate the Parameter Object here
            WITH loParamObj
                .AddProperty( 'nErrorCount', lnErrCnt )
                .AddProperty( 'aErrors[1]', "" )
                *** Now Get the Logged Details
                FOR lnCnt = 1 TO lnErrCnt
                    *** Add a row to the error array
                    DIMENSION .aErrors[lnCnt, 3]
                    *** Log the error as Character Strings
                    .aErrors[lnCnt, 1] = TRANSFORM( This.AErrors[ lnCnt , 1] )    && Error Number
                    .aErrors[lnCnt, 2] = TRANSFORM( This.AErrors[ lnCnt , 2] )    && Error Message
                    .aErrors[lnCnt, 3] = TRANSFORM( This.AErrors[ lnCnt , 3] )    && Prog/Method
                NEXT
            ENDWITH
        ENDIF
        *** Clear the Error Collection
        DIMENSION This.AErrors[1,3]
        This.AErrors = ""
        *** Return Errors
        RETURN loParamObj
    ENDFUNC

ENDDEFINE

***********************************************************************
* Class......: VFPTest
* Purpose....: Test Error Logging and handling in VFP
***********************************************************************
DEFINE CLASS VFPTest AS objbase OLEPUBLIC

    *******************************************************************
    *** [E] ERROR(): Augment default Error logging behaviour
    *******************************************************************
    FUNCTION Error( nError, cMethod, nLine )
        *** Call Root class behavior
        DODEFAULT( nError, cMethod, nLine )
        *** This Subclass returns direct to calling progam
        *** If one was specified, otherwise to 'Master'
        IF NOT EMPTY( This.cCalledBy )
            lcRetTo = "RETURN TO " + This.cCalledBy + " -1"
        ELSE
            lcRetTo = "RETURN"
        ENDIF
        &lcRetTo
    ENDFUNC

    ********************************************************************
    *** [E] GENERROR(): Deliberately generate a couple of Errors
    ********************************************************************
    FUNCTION GenError(tcCallBy)
        *** Save the name of the calling program (if any)        
        This.SaveCallBy( tcCallBy )
        *** Now generate an error
        ERROR 202      && File not found
        *** And to be sure that we have exited immediately
        *** Generate a second error here 
        *** NOTE: We should NOT get this error!
        ERROR 1570     && Database is read-only. 
        RETURN
    ENDFUNC

    ********************************************************************
    *** [P] SAVECALLBY(): Saves Called By Name to Property
    ********************************************************************
    PROTECTED FUNCTION SaveCallBy(tcCallBy)
        IF VARTYPE( tcCallBy ) = "C" AND NOT EMPTY( tcCallBy )
            This.cCalledBy = ALLTRIM( tcCallBy )
        ENDIF
        RETURN
    ENDFUNC

ENDDEFINE

***********************************************************************
* Class......: COMTest
* Purpose....: Test Error Logging and handling in a COM Object
***********************************************************************
DEFINE CLASS COMTest AS objbase OLEPUBLIC

    *******************************************************************
    *** [E] ERROR(): Augment default Error logging behaviour
    *******************************************************************
    FUNCTION Error( nError, cMethod, nLine )
        *** Call Root class behavior
        DODEFAULT( nError, cMethod, nLine )
        *** This Subclass returns direct to calling progam
        COMRETURNERROR( 'Error', 'Text' )
    ENDFUNC

    ********************************************************************
    *** [E] GENERROR(): Deliberately generate a couple of Errors
    ********************************************************************
    FUNCTION GenError(tcCallBy)
        *** Save the name of the calling program (if any)        
        This.SaveCallBy( tcCallBy )
        *** Now generate an error
        ERROR 202      && File not found
        *** And to be sure that we have exited immediately
        *** Generate a second error here 
        *** NOTE: We should NOT get this error!
        ERROR 1570     && Database is read-only. 
        RETURN
    ENDFUNC

    ********************************************************************
    *** [P] SAVECALLBY(): Saves Called By Name to Property
    ********************************************************************
    PROTECTED FUNCTION SaveCallBy(tcCallBy)
        IF VARTYPE( tcCallBy ) = "C" AND NOT EMPTY( tcCallBy )
            This.cCalledBy = ALLTRIM( tcCallBy )
        ENDIF
        RETURN
    ENDFUNC

ENDDEFINE

